import { IsEmail, IsOptional, IsString } from 'class-validator';
import { getConnectionManager } from 'typeorm';
import { UserNotFoundError } from '../../errors/AuthError';
import { UsernameOrEmailNeededError } from '../../errors/UserErrors';
import { JwtCreator } from '../../jwtcreator';
import { User } from '../entities/User';

/**
 * TODO:
 */
export class CreateResetToken {
    /**
     * The username of the user that wants to reset their password.
     */
    @IsOptional()
    @IsString()
    username?: string;

    /**
     * The email address of the user that wants to reset their password.
     */
    @IsOptional()
    @IsEmail()
    @IsString()
    email?: string;


    /**
     * Create a password reset token based on this.
     */
    public async toResetToken(): Promise<any> {
        if (this.email === undefined && this.username === undefined) {
            throw new UsernameOrEmailNeededError();
        }
        let found_user = await getConnectionManager().get().getRepository(User).findOne({ relations: ['groups', 'permissions', 'actions'], where: [{ username: this.username }, { email: this.email }] });
        if (!found_user) {
            throw new UserNotFoundError();
        }

        found_user.refreshTokenCount = found_user.refreshTokenCount + 1;
        await getConnectionManager().get().getRepository(User).save(found_user);

        //Create the reset
        let reset_token = JwtCreator.createReset(found_user);
        return reset_token;
    }
}